// avi_info.cpp: this utility analyzes the current avi file and output
//		related information, requires Video for Windows. (VFW)

#include <stdio.h>
#include <windows.h>
#include <vfw.h>

char *DWORD_to_fourCC(DWORD dw, char *fourcc);
STDAPI AVIFileException(PAVISTREAM pavi, PAVIFILE pfile);

int main(int argc, char *argv[])
{
	// declaring variables
	PAVIFILE avi_file;				// AVI file pointer
	PAVISTREAM avi_stream;			// AVI stream pointer
	AVISTREAMINFO avi_sinfo;		// AVI stream info structure;

	HRESULT hres;					// windows result handle
	char fourcc[4];					// fourCC string

	// parse command line
	if (argc != 2) {
		fprintf(stderr, "Usage: avi_info avi_file\n");
		exit(1);
	}
	else {
		// initialize AVI file functions and open input
		AVIFileInit();
		if ( FAILED(hres=AVIFileOpen(&avi_file, argv[1], OF_READ, NULL)) ) {
			fprintf(stderr, "Can not open AVI file %s\n", argv[1]);
			exit(2);
		}
		// get AVI stream interface
		if ( FAILED(hres=AVIFileGetStream(avi_file, &avi_stream, 0, 0)) ) {
			AVIFileException(avi_stream, avi_file);
			fprintf(stderr, "Can not get AVI stream interface\n");
			exit(2);
		}
	}

	// get AVI stream information structure
	ZeroMemory(&avi_sinfo, sizeof(avi_sinfo));
	if ( FAILED(hres=AVIStreamInfo(avi_stream, &avi_sinfo, sizeof(avi_sinfo)))) {
		AVIFileException(avi_stream, avi_file);
		fprintf(stderr, "Can not get AVI stream information\n");
		exit(3);
	}
	
	// print out AVI stream information
	printf("\nAVI file name\t\t\t%s\n", argv[1]);
	printf("Stream Description\t\t%s\n", avi_sinfo.szName);
	printf("Frame size\t\t\t%ld x %ld\n",
		avi_sinfo.rcFrame.right - avi_sinfo.rcFrame.left,
		avi_sinfo.rcFrame.bottom - avi_sinfo.rcFrame.top);
	printf("Sequence fourCC code\t\t%s\n",
		DWORD_to_fourCC(avi_sinfo.fccHandler, fourcc));
	printf("Sequence fourCC Type\t\t%s\n", 
		DWORD_to_fourCC(avi_sinfo.fccType, fourcc));
	printf("Sequence play-back rate\t\t%.3f frames/sec\n",
		(double)avi_sinfo.dwRate / avi_sinfo.dwScale);
	printf("Editing has been done\t\t%ld times\n", avi_sinfo.dwEditCount);
	printf("Format has been changed\t\t%ld times\n",
		avi_sinfo.dwFormatChangeCount);
	printf("Sequence start frame\t\t%ld\n", avi_sinfo.dwStart);
	printf("Sequence Length\t\t\t%ld frames\n", avi_sinfo.dwLength);
	printf("Suggest buffer size\t\t%ld bytes\n",
		avi_sinfo.dwSuggestedBufferSize);
	printf("Sequence Quality\t\t%ld\n", avi_sinfo.dwQuality);
	printf("Sample size\t\t\t%ld\n\n", avi_sinfo.dwSampleSize);
	printf("dwFlags\t\t\t\t%ld\n", avi_sinfo.dwFlags);
	printf("dwCaps\t\t\t\t%ld\n", avi_sinfo.dwCaps);
	printf("wPriority\t\t\t%ld\n", avi_sinfo.wPriority);
	printf("wLanguage\t\t\t%ld\n", avi_sinfo.wLanguage);

	// clean up and exit
	AVIFileException(avi_stream, avi_file);
	return(0);

}

// this function converts DWORD to fourcc string represenation
char *DWORD_to_fourCC(DWORD dw, char *fourcc)
{
	fourcc[0] = (CHAR) ( dw );
	fourcc[1] = (CHAR) ( dw >> 8 );
	fourcc[2] = (CHAR) ( dw >> 16 );
	fourcc[3] = (CHAR) ( dw >> 24 );

	return(fourcc);
}

STDAPI AVIFileException(PAVISTREAM pavi, PAVIFILE pfile)
{
	AVIStreamRelease(pavi);
	AVIFileRelease(pfile);
	AVIFileExit();
	return(0);
}