#include <stdio.h>
#include <string.h>
#include <windows.h>
#include <vfw.h>

#define TYPE_GRAY	0
#define TYPE_UYVY	1
#define TYPE_YV12	2

STDAPI AVIFileException(PAVISTREAM pavi, PAVIFILE pfile);

int main(int argc, char *argv[])
{
	// declaring variables
	PAVIFILE avi_file;				// AVI file pointer
	PAVISTREAM avi_stream;			// AVI stream pointer
	AVISTREAMINFO avi_sinfo;		// AVI stream info structure;
	BITMAPINFOHEADER bi_hdr;		// Bitmap info header structure;

	HRESULT hres;					// windows result handle

	char filename[80];
	char index[4];

	int width, height;				// frame width and height
	int fourCC;						// fourCC flag
	DWORD bsize, fsize;				// buffer size
	long first, count;
	long nBytes, nSamples;
	register long i, j;

	unsigned char *frame;			// frame buffer
	unsigned char *buffer;			// read in buffer
	unsigned char *u, *v;
	unsigned char ch;

	FILE *yuv_file;

	// parsing command line
	if ( argc != 7 ) {
		fprintf(stderr, 
			"Usage: yuv2avi yuvbase first count width height fourCC\n");
		fprintf(stderr, "\tsupported fourCC: UYVY, GRAY, YV12\n");
		exit(1);
	}
	else {
		// getting parameters
		if ( (first=atol(argv[2])) < 0 || (count=atol(argv[3])) <= 0 ||
			(width=atoi(argv[4])) <= 0 || (height=atoi(argv[5])) <=0 ) {
			fprintf(stderr, "Invalid count or size parameters\n");
			exit(1);
		}
		// check if specified fourCC code is supported
		if ( strcmp(argv[6], "UYVY") && strcmp(argv[6], "GRAY") &&
			 strcmp(argv[6], "YV12") ) {
			fprintf(stderr, "fourCC code %s is not supported\n", argv[6]);
			exit(1);
		}
	}

	// creating fourCC independent parameters
	avi_sinfo.fccType = streamtypeVIDEO;
	avi_sinfo.dwFlags = 0;			// containing AVITF_ flags
	avi_sinfo.dwCaps = 0;
	avi_sinfo.wPriority = 0;
	avi_sinfo.wLanguage = 0;
	avi_sinfo.dwScale = 33367;
	avi_sinfo.dwRate = 1000000;
	avi_sinfo.dwStart = first;
	avi_sinfo.dwLength = count;
	avi_sinfo.dwInitialFrames = 0;
	avi_sinfo.dwQuality = 0;
	avi_sinfo.dwSampleSize = 0;
	avi_sinfo.rcFrame.left = 0;
	avi_sinfo.rcFrame.top = 0;
	avi_sinfo.rcFrame.right = width;
	avi_sinfo.rcFrame.bottom = height;
	avi_sinfo.dwEditCount = 0;
	avi_sinfo.dwFormatChangeCount = 0;
	strcpy(avi_sinfo.szName, argv[1]);

	bi_hdr.biSize = sizeof(bi_hdr);
	bi_hdr.biWidth = width;
	bi_hdr.biHeight = height;
	bi_hdr.biPlanes = 1;
	bi_hdr.biXPelsPerMeter = 0;
	bi_hdr.biYPelsPerMeter = 0;
	bi_hdr.biClrUsed = 0;
	bi_hdr.biClrImportant = 0;

	// creating fourCC dependent parameters
	if ( !strcmp(argv[6], "UYVY") ) {
		// fourCC code UYVY
		avi_sinfo.fccHandler = mmioFOURCC('U','Y','V','Y');
		fsize = width * height;				// avi physical frame size
		bsize = width * height * 2;			// frame buffer size
		avi_sinfo.dwSuggestedBufferSize = bsize + 1024;
		bi_hdr.biBitCount = 16;				// every WORD a pixel
		bi_hdr.biSizeImage = bsize;
		bi_hdr.biCompression = mmioFOURCC('U','Y','V','Y');
		fourCC = TYPE_UYVY;
		// allocate memory
		if ( !(frame=(unsigned char *)malloc(sizeof(unsigned char)*bsize)) ||
			 !(buffer=(unsigned char *)malloc(sizeof(unsigned char)*bsize))) {
			fprintf(stderr, "Can not allocate memory\n");
		exit(3);		
		}
	}
	else if ( !strcmp(argv[6], "GRAY") ) {
		// fourCC code GRAY
		avi_sinfo.fccHandler = mmioFOURCC('U','Y','V','Y');
		fsize = width * height;				// avi physical frame size
		bsize = width * height * 2;			// frame buffer size
		avi_sinfo.dwSuggestedBufferSize = bsize + 1024;
		bi_hdr.biBitCount = 16;				// every WORD a pixel
		bi_hdr.biSizeImage = bsize;
		bi_hdr.biCompression = mmioFOURCC('U','Y','V','Y');
		fourCC = TYPE_GRAY;
		// allocate memory
		if ( !(frame=(unsigned char *)malloc(sizeof(unsigned char)*bsize)) ||
			 !(buffer=(unsigned char *)malloc(sizeof(unsigned char)*fsize))) {
			fprintf(stderr, "Can not allocate memory\n");
		exit(3);
		}
	}
	else if ( !strcmp(argv[6], "YV12") ) {
		// fourCC code YV12
		avi_sinfo.fccHandler = mmioFOURCC('Y','V','1','2');
		fsize = width * height;				// avi physical frame size
		bsize = width * height * 3 / 2;		// frame buffer size
		avi_sinfo.dwSuggestedBufferSize = bsize + 2048;
		bi_hdr.biBitCount = 12;				// every WORD a pixel
		bi_hdr.biSizeImage = bsize;
		bi_hdr.biCompression = mmioFOURCC('Y','V','1','2');
		fourCC = TYPE_YV12;
		// allocate memory
		if ( !(frame=(unsigned char *)malloc(sizeof(unsigned char)*bsize)) ||
			 !(buffer=(unsigned char *)malloc(sizeof(unsigned char)*bsize))) {
			fprintf(stderr, "Can not allocate memory\n");
		exit(3);
		}		
	}
	else {
		fprintf(stderr, "Unsupported fourCC\n");
		exit(2);
	}

	// initialize AVI file creation
	AVIFileInit();
	strcpy(filename, argv[1]);
	strcat(filename, ".avi");
	// create file
	if ( FAILED(hres=
		AVIFileOpen(&avi_file, filename, OF_CREATE | OF_WRITE, NULL)) ) {
		AVIFileExit();
		fprintf(stderr, "Can not create %s for output\n", filename);
		exit(1);
	}
	// create stream
	if ( FAILED(hres=AVIFileCreateStream(avi_file, &avi_stream, &avi_sinfo)) ) {
		AVIFileRelease(avi_file);
		AVIFileExit();
		fprintf(stderr, "Can not create AVI stream\n");
		exit(2);
	}
	// set stream format
	if ( FAILED(hres=
		AVIStreamSetFormat(avi_stream, 0, &bi_hdr, sizeof(bi_hdr))) ) {
		AVIFileException(avi_stream, avi_file);
		fprintf(stderr, "Can not set AVI stream format\n");
		exit(2);
	}

	// now start writing the frames
	for (i=first;i<first+count;i++) {

		// create input file name
		sprintf(index, "%03d", i);
		strcpy(filename, argv[1]);
		strcat(filename, index);
		if ( fourCC == TYPE_GRAY )
			strcat(filename, ".raw");
		else 
			strcat(filename, ".yuv");
			
		// open input file
		if ( (yuv_file=fopen(filename, "rb")) == NULL ) {
			AVIFileException(avi_stream, avi_file);
			fprintf(stderr, "Can not open YUV file %s for input\n", filename);
			exit(4);
		}

		// switch based on fourCC code
		switch(fourCC) {

		case TYPE_UYVY:
			if (fread(frame, sizeof(unsigned char), bsize, yuv_file)!=bsize)
				fprintf(stderr, "file %s size is not %d\n", filename, bsize);
			fclose(yuv_file);
			if ( FAILED(hres=
				AVIStreamWrite(avi_stream, i-first, 1, frame, bsize,
				AVIIF_KEYFRAME,	&nSamples, &nBytes)) ) {
				AVIFileException(avi_stream, avi_file);
				fprintf(stderr, "Frame write error at %d\n", i);
				exit(5);
			}
			break;

		case TYPE_YV12:
			if (fread(frame, sizeof(unsigned char), bsize, yuv_file)!=bsize)
				fprintf(stderr, "file %s size is not %d\n", filename, bsize);
			fclose(yuv_file);
			// swap U and V field
			u = frame + fsize;
			v = frame + fsize * 5 / 4;
			for (j=0;j<fsize/4;j++) {
				ch = u[j];
				u[j] = v[j];
				v[j] = ch;
			}
			if ( FAILED(hres=
				AVIStreamWrite(avi_stream, i-first, 1, frame, bsize,
				AVIIF_KEYFRAME,	&nSamples, &nBytes)) ) {
				AVIFileException(avi_stream, avi_file);
				fprintf(stderr, "Frame write error at %d\n", i);
				exit(5);
			}
			break;

		case TYPE_GRAY:
			if (fread(buffer, sizeof(unsigned char), fsize, yuv_file)!=fsize)
				fprintf(stderr, "file %s size is not %d\n", filename, fsize);
			fclose(yuv_file);
			memset(frame, 128, bsize);
			for (j=0;j<fsize;j++)
				frame[j*2+1] = buffer[j];
			if ( FAILED(hres=
				AVIStreamWrite(avi_stream, i-first, 1, frame, bsize,
				AVIIF_KEYFRAME,	&nSamples, &nBytes)) ) {
				AVIFileException(avi_stream, avi_file);
				fprintf(stderr, "Frame write error at %d\n", i);
				exit(5);
			}
			break;

		default:
			break;
		}
	}

	free(frame);
	free(buffer);
	AVIFileException(avi_stream, avi_file);
	return(0);

}

STDAPI AVIFileException(PAVISTREAM pavi, PAVIFILE pfile)
{
	AVIStreamRelease(pavi);
	AVIFileRelease(pfile);
	AVIFileExit();
	return(0);
}